      subroutine readoutgrid()

********************************************************************************
* $Id: readoutgrid.f,v 1.1 2006/06/21 11:13:45 paul Exp paul $
* $Log: readoutgrid.f,v $
* Revision 1.1  2006/06/21 11:13:45  paul
* Initial revision
*
*                                                                              *
*     This routine reads the user specifications for the output grid.          *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     4 June 1996                                                              *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* dxout,dyout          grid distance                                           *
* numxgrid,numygrid,numzgrid    grid dimensions                                *
* outlon0,outlat0      lower left corner of grid                               *
* outheight(maxzgrid)  height levels of output grid [m]                        *
*                                                                              *
* Constants:                                                                   *
* unitoutgrid          unit connected to file OUTGRID                          *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer i,j
      real outhelp,xr,xr1,yr,yr1,eps
      parameter(eps=1.e-4)

      if (lc_or_lamphi) stop 'LC OUTGRID implementation not yet ready!'

C Open the OUTGRID file and read output grid specifications
***********************************************************

      open(unitoutgrid,file=path(1)(1:len(1))//'OUTGRID',status='old',
     +err=999)

      call skplin(5,unitoutgrid)

C 1.  Read horizontal grid specifications
*****************************************

      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f11.4)') outlon0
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f11.4)') outlat0
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,i5)') numxgrid
      if (numxgrid .gt. maxxgrid) stop 'Please increase maxxgrid'
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,i5)') numygrid
      if (numygrid .gt. maxygrid) stop 'Please increase maxygrid'
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f12.5)') dxout
      if(lc_or_lamphi) dxout=dxout*1000.
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f12.5)') dyout
      if (lc_or_lamphi) dyout=dyout*1000.
!! really, when using lc ??      if (dyout.ne.dxout) goto 995

C Check validity of output grid (shall be within model domain)
**************************************************************

      if (lc_or_lamphi) then
	  print '("OUTGRID is using Lambert-Conformal projection.")'
	else
	  print '("OUTGRID is using a regular lat/lon grid.")'
      endif
	
C Get Motherdomain extensions
      call ij_to_latlon(float(nxfield-1),float(ny-1),yr1,xr1)

      if (lc_or_lamphi) then
CPS this section is not checked with respect to ij-xy transformations!	
        call latlon_to_ij(outlat0,outlon0,xr,yr) 
        call map_set(code_sav,lat1_sav,lon1_sav,xr,yr,dxout,stdlon_sav,
     &        truelat1_sav,truelat2_sav)
!      call map_set(code_sav,outlat0,outlon0,xr,yr,dxout,stdlon_sav,
!     &truelat1_sav,truelat2_sav)
        outlat1_sav=outlat0
        outlon1_sav=outlon0
        dxout_sav=dxout
        kiout_sav=xr
        kjout_sav=yr

        do 992 j=1,numygrid
         do 992 i=1,numxgrid
          outlon(i,j)=-999.
          outlat(i,j)=-999.
992     continue
    
!      call ij_to_latlon(float(numxgrid),float(numygrid),yr,xr)
        do 991 j=1,numygrid
          do 991 i=1,numxgrid
            call ij_to_latlon(float(i),float(j),yr,xr) 
            outlon(i-1,j-1)=xr
            outlat(i-1,j-1)=yr
 991    continue
        print '("Outgrid domain:",1/,"  Longitude range: ",f10.1," 
     &  ",f10.1,"   Grid distance (ca. deg/meter): ",f10.4,"/",f10.4)
     &  ',outlon0,xr,(outlon0+xr)/float(numxgrid),dxout
            
        print '("  Latitude range:  ",f10.1," ",
     &   f10.1,"   Grid distance (ca. deg/meter): ",f10.4,"/",f10.4,1/)'
     &           ,outlat0,yr,(outlat0+yr)/float(numygrid),dyout

      else ! lamphi output grid
	
        xr=outlon0+float(numxgrid)*dxout
        yr=outlat0+float(numygrid)*dyout
        outlat1_sav=outlat0
        outlon1_sav=outlon0
        print '("Outgrid domain:",1/,"  Longitude range: ",f10.1," ",
     &    f10.1,"   Grid distance (deg): ",f10.4)',outlon0,xr,dxout
            
        print '("  Latitude range:  ",f10.1," ",f10.1,
     &	  "   Grid distance (deg): ",f10.4,1/)',outlat0,yr,dyout
      
	endif

C Set Map back to MM5 input files
      call map_set(code_sav,lat1_sav,lon1_sav,1.,1.,dx_sav,
     &           stdlon_sav,truelat1_sav,truelat2_sav)

      if (outlon0+eps .lt. xlon0 .or. outlat0+eps .lt. ylat0
     &  .or. xr .gt. xr1+eps .or. yr .gt. yr1+eps) then
        write(*,*) ' #### FLEXPART MODEL ERROR! PART OF OUTPUT    ####'
        write(*,*) ' #### GRID IS OUTSIDE MODEL DOMAIN. CHANGE    ####'
        write(*,*) ' #### FILE OUTGRID IN DIRECTORY               ####'
        write(*,'(a)') path(1)(1:len(1))
        stop
      endif
      if (numxgrid .gt. maxxgrid .or. numygrid.gt.maxygrid) then
        write(*,*) ' #### FLEXPART MODEL ERROR! DIMENSIONS OF     ####'
        write(*,*) ' #### OUTPUT GRID EXCEED MAXIMUM VALUES.      ####'
        write(*,*) ' #### CHANGE FILE $FLEXPART/options/OUTGRID.  ####'
        stop
      endif

C 2. Vertical levels of output grid
***********************************

      j=0
100     j=j+1
        do 20 i=1,3
20        read(unitoutgrid,*,end=99)
        read(unitoutgrid,'(4x,f7.1)',end=99) outhelp
        if (outhelp.eq.0.) goto 99
        if (j.gt.maxzgrid) then
        write(*,*) ' #### FLEXPART MODEL ERROR! TOO MANY HEIGHT   #### ' 
        write(*,*) ' #### LEVELS ARE GIVEN FOR OUTPUT GRID.       #### ' 
        write(*,*) ' #### MAXIMUM NUMBER IS ',maxzgrid,'          #### ' 
        write(*,*) ' #### PLEASE MAKE CHANGES IN FILE OUTGRID.    #### ' 
        endif
        outheight(j)=outhelp
        goto 100
99    numzgrid=j-1


C Check whether vertical levels are specified in ascending order
****************************************************************

      do 40 j=2,numzgrid
        if (outheight(j).le.outheight(j-1)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! YOUR SPECIFICATION#### ' 
        write(*,*) ' #### OF OUTPUT LEVELS IS CORRUPT AT LEVEL    #### ' 
        write(*,*) ' #### ',j,'                              #### ' 
        write(*,*) ' #### PLEASE MAKE CHANGES IN FILE OUTGRID.    #### ' 
        endif
40      continue

C Determine the half levels, i.e. middle levels of the output grid
******************************************************************

      outheighthalf(1)=outheight(1)/2.
      do 50 j=2,numzgrid
50      outheighthalf(j)=(outheight(j-1)+outheight(j))/2.

      xoutshift=xlon0-outlon0
      youtshift=ylat0-outlat0

      close(unitoutgrid)
      return

995   write(*,*) ' #### FLEXPART MODEL ERROR! GRIDRESOLUTION IN #### '
      write(*,*) ' #### FILE "OUTGRID" MUST BE THE SAME.        #### '
      stop

999   write(*,*) ' #### FLEXPART MODEL ERROR! FILE "OUTGRID"    #### '
      write(*,*) ' #### CANNOT BE OPENED IN THE DIRECTORY       #### '
      write(*,*) ' #### xxx/flexpart/options                    #### '
      stop

      end
